plugins {
    java
    application
    jacoco
    id("cb-artifactory-conventions")
    id("com.carbonblack.plugins.versioning")
    // Linting (./gradlew spotlessApply or ./gradlew spotlessCheck)
    id("com.diffplug.spotless")
    // Generate asciidoc docs
    id("org.asciidoctor.jvm.convert")
    // Support lombok
    id("io.freefair.lombok")
    // Analyze dependencies and provide recommendations (./gradlew audit)
    id("net.ossindex.audit")
    // Test coverage logging
    id("org.barfuin.gradle.jacocolog")
    // Log tests as they happen for nicer CI
    id("com.adarshr.test-logger")
    // helps to create spring boot jar
    id("org.springframework.boot")
}

group = "com.vmware.psc.alerting.api"
version = "0.0.1"

application {
    mainClass.set("com.vmware.psc.alerting.api.AlertApiServiceApplication")
}

// Main dependencies
dependencies {
    implementation("org.slf4j:slf4j-api")
    implementation("net.logstash.logback:logstash-logback-encoder:7.3")
    implementation("com.fasterxml.jackson.core:jackson-databind")
    implementation("com.google.code.gson:gson:2.10")

    implementation("org.springframework.boot:spring-boot-starter")
    implementation("org.springframework.boot:spring-boot-starter-actuator") {
        exclude(group="org.yaml", module="snakeyaml")
    }
    implementation("org.springframework.boot:spring-boot-starter-web")
    implementation("org.springframework.boot:spring-boot-starter-validation")
    implementation("org.springframework.boot:spring-boot-starter-security")
    implementation("org.springframework.boot:spring-boot-starter-aop")
    implementation("org.springframework.boot:spring-boot-configuration-processor")
    annotationProcessor("org.springframework.boot:spring-boot-configuration-processor:3.2.2")

    implementation("org.owasp.encoder:encoder:1.2.3")

    implementation("org.springdoc:springdoc-openapi-starter-webmvc-ui:2.2.0")

    implementation("com.coveo:spring-boot-parameter-store-integration:1.5.0")
    implementation("io.micrometer:micrometer-registry-wavefront")
    implementation("com.amazonaws:aws-java-sdk-core")
    implementation("software.amazon.awssdk:sqs")
    implementation("com.amazonaws:aws-java-sdk-sts")
    implementation("com.amazonaws:aws-java-sdk-sqs")
    implementation("software.amazon.kinesis:amazon-kinesis-client:2.5.4")
    implementation("com.amazonaws:amazon-kinesis-producer:0.15.8") {
        exclude(group="io.netty", module="netty-codec")
    }

    implementation("software.amazon.awssdk:auth")

    implementation("com.google.guava:guava:33.0.0-jre")

    implementation("org.apache.lucene:lucene-spellchecker:3.6.2")

    // to support forwarding configurable host headers
    implementation("com.vmware.psc:psc-api-client:0.2.0") {
        exclude(group= "com.carbonblack.psc.security", module ="spring-auth-middleware")
        exclude(group= "com.carbonblack.psc.security", module ="access-service-client")
    }
    implementation("com.carbonblack.psc.security:spring-auth-middleware:1.0.3") {
        exclude(group= "com.carbonblack.psc.apicore", module = "common")
    }
    implementation("com.carbonblack.psc.security:access-service-client:1.0.3") {
        exclude(group= "com.carbonblack.psc.apicore", module = "common")
    }
    // to support spring boot 3 migration - upgrade to use jakarta validation
    implementation("com.carbonblack.psc.apicore:common:0.3.3")
    implementation("com.vmware.psc:time-helper:0.0.1")
    implementation("com.carbonblack.psc.jobs:jobs-client:1.3.8")
    implementation("com.carbonblack.psc:org-key:1.5.2")
    implementation("com.carbonblack.psc.alerts:alert-model:0.11.31")
    implementation("com.carbonblack.psc:exponential-backoff:2.1.0")
    // HEY! Before you update this version, have you:
    //   Updated the mappings in es-config (and mappings-generator, if necessary)?
    //   Deployed the updated mappings to all envs?
    implementation("com.carbonblack.protocols.psc.response:PscRProto:1.9.3607")
    implementation("com.carbonblack.psc_r:pscr-compressors:main.95bec5d")
    implementation("com.vmware.psc:data-helpers:0.0.2")
    implementation("com.vmware.psc.alerting:alert-data-mapper:0.2.12")
    implementation("com.vmware.psc.alerting:common-es-utils:0.0.1")
    implementation("org.opensearch:opensearch:2.7.0")

    // Lock our ES library version to the version actually running in AWS
    implementation("org.opensearch.client:opensearch-rest-high-level-client") {
        version {
            strictly("2.11.1")
        }
    }


    implementation("com.scargo:domainModel:1.19.0") {
        exclude(group = "com.carbonblack.psc.alerts", module = "alert-model")
        exclude(group = "org.jsoup", module = "jsoup")
        exclude(group = "org.hibernate", module = "hibernate-validator")
    }
    // TODO - this seems like it's from the CBD build.
    // Once the builds stabalize on https://gitlab.bit9.local/cbc-identity/audit-log-service we
    // should update to use one of those tags instead.
    implementation("com.scargo:audit-log-client:1.12.6")
    implementation("org.projectlombok:lombok")
    annotationProcessor("org.projectlombok:lombok")
    implementation("org.mapstruct:mapstruct:1.5.5.Final")
    annotationProcessor("org.mapstruct:mapstruct-processor:1.5.5.Final")
    annotationProcessor("org.projectlombok:lombok-mapstruct-binding:0.2.0")
}

// Manual transitive dependency upgrades for vulnerable or "vulnerable" libraries
dependencies {
    // Brought in by spring
    // Not a real vulnerability but it's easier to just force the upgrade than fix the scan
    implementation("org.yaml:snakeyaml:2.0")

    // Brought in by kinesis client lib
    implementation("org.json:json:20231013")

    // Strictly speaking not necessary anymore since we don't use this compression method for UDS.
    // But we do see it brought in as a transitive dep from the aws kinesis library so updating it here.
    implementation("org.xerial.snappy:snappy-java:1.1.10.4")

    // Brought in by Micrometer/aws
    implementation("org.apache.avro:avro:1.11.3")
    implementation("org.apache.commons:commons-compress:1.26.0")

    // Mutiple places, such as aws sdk, spring middleware
    implementation("io.netty:netty-codec:4.1.104.Final")

    implementation(platform("com.fasterxml.jackson:jackson-bom:2.16.1"))

    implementation("org.springframework.security:spring-security-core:6.2.3")

    implementation("org.springframework:spring-web:6.1.5")
}

// Caching dependencies
dependencies {
    implementation("org.springframework.boot:spring-boot-starter-cache")
    implementation("javax.cache:cache-api")
    // to support Jakarta EE 9 or later (required by Spring 3), need to declare with a jakarta classifier
    implementation("org.ehcache:ehcache:3.10.8:jakarta")
}

// Test dependencies
dependencies {
    testImplementation("com.vmware.psc.alerting:sample-data:0.2.2")
    testImplementation("org.springframework.boot:spring-boot-starter-test") {
        exclude(group = "org.junit.vintage", module = "junit-vintage-engine")
        exclude(group = "com.vaadin.external.google", module = "android-json")
    }
    testImplementation("org.springframework.security:spring-security-test")
    testImplementation("org.junit.jupiter:junit-jupiter-api")
    testImplementation("org.awaitility:awaitility")

    testImplementation("commons-io:commons-io:2.13.0")
    testImplementation("org.apache.commons:commons-lang3:3.13.0")
    testRuntimeOnly("org.junit.jupiter:junit-jupiter-engine")

    testImplementation("org.springframework.boot:spring-boot-test")
    testImplementation("org.awaitility:awaitility-kotlin:4.2.0")
}

// For applications, we enable dependency locking.
// See: https://docs.gradle.org/current/userguide/dependency_locking.html
dependencyLocking {
   lockAllConfigurations()
}

// Platform Dependencies (BOMs in Maven-land) specify ranges for several
// sub projects.
dependencies {
    implementation(platform("org.springframework.boot:spring-boot-dependencies:3.2.3"))
    implementation(platform("com.amazonaws:aws-java-sdk-bom:1.12.633"))
    implementation(platform("software.amazon.awssdk:bom:2.22.13"))
    implementation(platform("io.awspring.cloud:spring-cloud-aws-dependencies:3.0.1"))
}

// Other dependencies are often selected via range selectors.
// With version locking, updates to lockfiles are made only when you
// decide to update the lockfiles.
dependencies.constraints {
    testImplementation("org.junit.jupiter:junit-jupiter-api:5.10.1")
    testRuntimeOnly("org.junit.jupiter:junit-jupiter-engine:5.10.1")
}

// to fix the duplicate jaxb-core jar error (org.glassfixh.jaxb and com.sun.xml,bind) in gradle dist tasks (bootJar,
// distZip, etc) - Spring 3 uses the Glassfish RI so exclude the duplicate com.sun.xml.bind jaxb-core dependency
configurations.all {
    exclude(group= "com.sun.xml.bind")
    exclude(group = "io.springfox")
    exclude(group = "com.vaadin.external.google", module = "android-json")
}

// Configure a separate sourceset for integration testing.
// Someday the incubating JVM Test Suite plugin will reach maturity and we can use it instead of this manual configuration
sourceSets {
    create("intTest") {
        compileClasspath += sourceSets.main.get().output
        runtimeClasspath += sourceSets.main.get().output
    }
}

val intTestImplementation by configurations.getting {
    extendsFrom(configurations.implementation.get())
}
val intTestRuntimeOnly by configurations.getting

configurations["intTestRuntimeOnly"].extendsFrom(configurations.runtimeOnly.get())

tasks.named<Test>("test") {
    useJUnitPlatform()
    finalizedBy(tasks.jacocoTestReport)
}

val integrationTest = task<Test>("integrationTest") {
    description = "Runs integration tests."
    group = "verification"

    testClassesDirs = sourceSets["intTest"].output.classesDirs
    classpath = sourceSets["intTest"].runtimeClasspath

    useJUnitPlatform()

    exclude("com/vmware/psc/alerting/api/integration/standards/**")
}

val integrationTestLint = task<Test>("integrationTestLint") {
    description = "Lint integration tests."
    group = "verification"

    testClassesDirs = sourceSets["intTest"].output.classesDirs
    classpath = sourceSets["intTest"].runtimeClasspath

    useJUnitPlatform()

    testLogging {
        events("passed")
    }

    include("com/vmware/psc/alerting/api/integration/standards/**")

}

dependencies {
    intTestImplementation("org.junit.jupiter:junit-jupiter:5.10.1")
    intTestRuntimeOnly("org.junit.platform:junit-platform-launcher")
    intTestImplementation("org.springframework.boot:spring-boot-starter-web")
    intTestImplementation("org.springframework.boot:spring-boot-starter-webflux")
    intTestImplementation("org.springframework.boot:spring-boot-starter-test") {
        exclude(group = "org.junit.vintage", module = "junit-vintage-engine")
    }
    intTestImplementation("com.tngtech.archunit:archunit-junit5:1.2.0")
    intTestImplementation("io.github.artsok:rerunner-jupiter:2.1.6")
    intTestImplementation("net.datafaker:datafaker:2.0.2")
    intTestImplementation("com.vmware.psc.alerting:sample-data:0.3.1")
}

jacoco {
    toolVersion = "0.8.8"
}

tasks.jacocoTestReport {
    dependsOn(tasks.test)
    reports {
        xml.required.set(true)
    }
}

// The spotless plugin needs to match the version in use by IntellIJ
spotless {
    java {
        googleJavaFormat("1.16.0")
    }
}

// Our Spring Boot deployments generally recommend a recent
// GA release of OpenJDK, which package the slimmed down JRE, and
// provide a lot of performance improvements.
java.sourceCompatibility = JavaVersion.VERSION_17

// Customize our asciidoc distribution.
//
// Includes source highlighting and TOC auto-expansion via tocbot, which is
// configured in docinfo.
asciidoctorj {
}

tasks {
    val asciidoctor by named("asciidoctor", org.asciidoctor.gradle.jvm.AsciidoctorTask::class)
    asciidoctor.attributes(mapOf(
            "source-highlighter" to "rouge",
            "toc" to "left",
            "docinfo" to "shared"
    ))
    asciidoctor.resources {
        from("src/docs/asciidoc") {
            include("**/*.png")
        }
    }
}

tasks.named("compileJava").configure {
    dependsOn(tasks.named("spotlessApply"))
}
